/**
 * @file main.c
 *
 *  Created on: Dec 7, 2009
 *  Author: Ruediger Knoerig
 */

#ifndef F_CPU
#warning "F_CPU war noch nicht definiert, wird nun mit 7,3728 MHz definiert"
#define F_CPU 7372800UL   
#endif

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h> 
#include <avr/sleep.h>
#include "DigSinGenerator.h"

#define OUTPUT (1<<PB3)
#define TASTER1 (1<<PD2)
#define TASTER2 (1<<PD3)

// Frequency of the Sinus relative to the sampling frequency
volatile double f=0.05;
// Sinus generation object
DigSinGenerator sinus(f);

/**
 * @brief Interrupt service routine to handle keypress.
 * This ISR decreases the frequency of the sinus.
 */
ISR(INT0_vect)
{
  uint8_t tmp_sreg;  // temporaerer Speicher fuer das Statusregister
  tmp_sreg = SREG;
  cli();
  if(f > 0.05) f-=0.05;
  sinus.setFrequency(f);
  _delay_ms(10);
  sei();
  SREG=tmp_sreg;
}

/**
 * @brief Interrupt service routine to handle keypress.
 * This ISR increases the frequency of the sinus.
 */
ISR(INT1_vect)
{
  uint8_t tmp_sreg;  // temporaerer Speicher fuer das Statusregister
  tmp_sreg = SREG;
  cli();
  if(f < 0.5) f+=0.05;
  sinus.setFrequency(f);
  _delay_ms(10);
  sei();
  SREG=tmp_sreg;
}

/**
 * @brief Timer 2 Compare A interrupt service routine.
 * Used to generate the sampling clock.
 */
ISR(TIMER2_COMP_vect)
{
  // Folgende Zeile Sägezahn zum Testen
  //OCR0++; 
  OCR0=sinus();
}

int main()
{
  DDRB = OUTPUT;
  PORTD = TASTER1 | TASTER2; /* activate pull-up resistors */
  GICR = (1<<INT0) | (1<<INT1); /* activate INT0/1 interrupt on falling edge */
  OCR0 = 0;
  // enable non-inverted PWM mode for timer 0
  // set prescaler of Timer0 to 1
  TCCR0 = (1<<WGM01) | (1<<WGM00) | (1<<COM01) | (1<<CS00);
  OCR2 = 1; // f_T=28.8 kHz with fosc=7.3728 MHz
  // Timer2 in CTC mode, prescaler = 64
  TCCR2 = (1<<WGM21) | (1<<CS22);
  // Enable timer2 compare inerrupt
  TIMSK = (1<<OCIE2);
  sei();
  while(1)
    {
      set_sleep_mode(SLEEP_MODE_IDLE);
      sleep_mode();
    }
}
